unit formulae;



interface

uses math,complex;

type return = record
                  x,y,z:float;
                  i:integer;
              end;


function interpolate(i1,i2:integer):integer;


function mandelbrot(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;

function spider(ca,cbi:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;

function barnsley(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;

function div1zc(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;

function logsin(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function mandelorder(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function sinzpz(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function ikenaga(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function webb(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function webbII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function newtonI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function costnewton(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function phoenix(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function nova(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function newton_mI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function newton_mII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function novavarI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function novavarII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function newtonmIII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function newtonmiv(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function quartetI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;

function quartetII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;




function newtest(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;




implementation

//----------> MISC

function interpolate(i1,i2:integer):integer;
begin
interpolate:=round((i1+i2)/2);
end;

function cflip(c1:tcomplex):tcomplex;
begin
cflip.x:=c1.y;
cflip.y:=c1.x;
end;

function fzehn_hoch(x : double): double; assembler;
const round_down : word = $177F;
var control_ww : word;
asm                   {   1         2        3    }
   FLD X              {  x         -         -    }
   FLDL2T             { log2(10)   x         -    }
   FMULP ST(1), ST    { x.log2(10) -         -    }
   FSTCW control_ww
   FLDCW round_down
   FLD ST(0)          {  z         z          -   }
   FRNDINT            { int(z)     z          -   }
   FLDCW control_ww
   FXCH               {  z         i          -   }
   FSUB ST, ST(1)     {  f         i          -   }
   F2XM1              { 2^f-1      i          -   }
   FLD1               {  1        2^f-1       i   }
   FADDP ST(1), ST    { 2^f        i          -   }
   FSCALE             { 2^f.2^i    i          -   }
   FSTP ST(1)         { 10^x       -          -   }
end;




//----------> MANDELBROT

function mandelbrot(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc{,temp,zp}:tcomplex;
   length_z:float;

BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi;
   c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;
   if not julia then z:=cmplx(0,0) else z:=c;

       if not julia then
       REPEAT
       z:=cadd(cmulti(z,z),c);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z:=cadd(cmulti(z,z),cc);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   mandelbrot.x:=z.x;
   mandelbrot.y:=z.y;
   mandelbrot.z:=length_z;
   mandelbrot.i:=iteration;
end;

//----------> SPIDER

function spider(ca,cbi:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;
var
    OLD_A,a,b,
    length_z       :float;    {just a variable to keep 'a' from being destroyed}
    ITERATION      :integer; {the iteration-counter}
    tmp:tcomplex;
BEGIN
  if invert then
  begin
  tmp.x:=ca;
  tmp.y:=cbi;
  ca:=cintdiv_RC(1,tmp).x;
  cbi:=cintdiv_RC(1,tmp).y;
  end;
     A:=ca; B:=cbi;
     ITERATION:=0;            {initialize iteration}
     REPEAT
      OLD_A:=a;             {saves the 'a'  (Will be destroyed in next line}
      A:= A*A - B*B + CA;
      B:= 2*old_a*B + CBi;
      ca:=ca/2+a;
      cbi:=cbi/2+b;
      length_z:= a*a + b*b;
      inc(iteration);
     UNTIL (length_z > bailout) OR (iteration > max_iteration);
   spider.x:=a;
   spider.y:=b;
   spider.z:=length_z;
   spider.i:=iteration;
end;

//----------> BARNSLEY

function barnsley(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;
var
    OLD_A,a,b,
    length_z       :float;    {just a variable to keep 'a' from being destroyed}
    ITERATION      :integer; {the iteration-counter}
    tmp            :tcomplex;
BEGIN
     if invert then
  begin
  tmp.x:=ca;
  tmp.y:=cbi;
  ca:=cintdiv_RC(1,tmp).x;
  cbi:=cintdiv_RC(1,tmp).y;
  end;
     if not julia then begin A:=0; B:=0; end else begin A:=ca; B:=cbi; end;
     ITERATION:=0;
     if not julia then
     REPEAT
      if a<0 then a:=a+1 else a:=a-1;
      old_a:=a;
      A:= a*ca-b*cbi;
      B:= old_a*cbi+b*ca;
      length_z:= a*a + b*b;
      inc(iteration);
     UNTIL (length_z > bailout) OR (iteration > max_iteration)
     else
     REPEAT
      if a<0 then a:=a+1 else a:=a-1;
      old_a:=a;
      A:= a*const_ext-b*const_imag;
      B:= old_a*const_imag+b*const_ext;
      length_z:= a*a + b*b;
      inc(iteration);
     UNTIL (length_z > bailout) OR (iteration > max_iteration);
   barnsley.x:=a;
   barnsley.y:=b;
   barnsley.z:=length_z;
   barnsley.i:=iteration;
end;

//----------> DIV1Z
function div1zc(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z_,c,cc,tmp:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;
   z:=c;
   tmp.x:=2;
   tmp.y:=0;
       if not julia then
       REPEAT
       z_:=z;
       z:=cdiv(cintpower(z_,2),cadd(tmp,z_));
       z_:=z;
       z:=cadd(z_,c);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z_:=z;
       z:=cdiv(cintpower(z_,2),cadd(tmp,z_));
       z_:=z;
       z:=cadd(z_,cc);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   div1zc.x:=z.x;
   div1zc.y:=z.y;
   div1zc.z:=length_z;
   div1zc.i:=iteration;

end;

//----------> logsin
function logsin(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z_,c,cc:tcomplex;
   length_z:float;

BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;
   z:=c;
       if not julia then
       REPEAT
       z_:=z;
       z:=cintpower(z_,order);
       z_:=z;
       z:=cadd(clog(csin(z_)),c);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z_:=z;
       z:=cintpower(z_,order);
       z_:=z;
       z:=cadd(clog(csin(z_)),cc);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   logsin.x:=z.x;
   logsin.y:=z.y;
   logsin.z:=length_z;
   logsin.i:=iteration;
end;

//----------> mandelorder
function mandelorder(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;

   z:=c;
       if not julia then
       REPEAT
       z:=cadd(cintpower(z,order),c);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z:=cadd(cintpower(z,order),cc);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   mandelorder.x:=z.x;
   mandelorder.y:=z.y;
   mandelorder.z:=length_z;
   mandelorder.i:=iteration;
end;

//----------> sinzpz
function sinzpz(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z_,c,cc:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;
   z:=c;
       if not julia then
       REPEAT
       z_:=z;
       z:=cadd(csin(cadd(cintpower(z,order),c)),z);
       inc(iteration);
       try length_z:=z.x*z.x+z.y*z.y except length_z:=0 end;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z_:=z;
       z:=cadd(csin(cadd(cintpower(z,order),cc)),z);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration) ;
   sinzpz.x:=z.x;
   sinzpz.y:=z.y;
   sinzpz.z:=length_z;
   sinzpz.i:=iteration;
end;

//----------> ikenaga
function ikenaga(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z_,c,cc:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;

       if not julia then
       REPEAT
       z:=cadd(cintpower(z,order),cmulti(z,csub_CR(c,1)));
       z:=cadd(z,c);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z:=cadd(cintpower(z,3),cmulti(z,csub_CR(cc,1)));
       z:=cadd(z_,cc);
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   ikenaga.x:=z.x;
   ikenaga.y:=z.y;
   ikenaga.z:=length_z;
   ikenaga.i:=iteration;
end;

//----------> webb
function webb(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2:tcomplex;
   length_z:float;

BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
   z2.x:=0;
   z2.y:=0;
   z1:=z;
       if not julia then
       REPEAT
       z1:=z;
       z:=cadd(csub(cintpower(z,order),z2),c);
       z2:=z1;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z1:=z;
       z:=cadd(csub(cintpower(z,order),z2),cc);
       z2:=z1;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   webb.x:=z.x;
   webb.y:=z.y;
   webb.z:=length_z;
   webb.i:=iteration;
end;

//----------> webbII
function webbII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2:tcomplex;
   length_z:float;

BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
   z2.x:=0;
   z2.y:=0;
   z1:=z;
       if not julia then
       REPEAT
       z1:=z;
       z:=cadd(cadd(cintpower(z,order),cintdiv_CR(z2,2)),c);
       z2:=z1;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z1:=z;
       z:=cadd(cadd(cintpower(z,order),cintdiv_CR(z2,2)),c);
       z2:=z1;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   webbII.x:=z.x;
   webbII.y:=z.y;
   webbII.z:=length_z;
   webbII.i:=iteration;
end;

//----------> newtonI
function newtonI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,z1,z2:tcomplex;
   length_z,delta:float;

BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     if invert then begin c:=cintdiv_RC(1,c); {cc:=cintdiv_RC(1,cc);}  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
   //z(n+1) = ((p-1)*z(n)^p + 1)/(p*z(n)^(p - 1)).

       REPEAT
       z1:=z;
       z:=cadd_CR(cmulti_CR(cintpower(z,order),order-1),1);
       z:=cdiv(z,cmulti_CR(cintpower(z1,order-1),order));
       inc(iteration);
       z2:=csub(z,z1);
       try length_z:=z2.x*z2.x+z2.y*z2.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration);

   newtonI.x:=z.x;
   newtonI.y:=z.y;
   newtonI.z:=length_z;
   newtonI.i:=iteration;
end;

//----------> costnewton
function costnewton(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2:tcomplex;
   length_z,delta:float;

BEGIN
   delta:=1/fzehn_hoch(bailout);
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
   z2.x:=0;
   z2.y:=0;
   z1:=z;
       REPEAT
       z1:=z;
       z:=cadd_CR(cmulti_CR(cintpower(z,order),order-1),1);
       z:=cdiv(z,cmulti_CR(cintpower(z1,order-1),order));

       z:=cadd(z,c);
       //z:=csub(z1,z);
       inc(iteration);
       z2:=csub(z,z1);
       //z2:=z;
       try length_z:=z2.x*z2.x+z2.y*z2.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration){ or (length_Z>bailout)};

   costnewton.x:=z.x;
   costnewton.y:=z.y;
   costnewton.z:=length_z;
   costnewton.i:=iteration;
end;

//----------> phoenix
function phoenix(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2:tcomplex;
   length_z:float;

BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
   z2.x:=0;
   z2.y:=0;
       if not julia then
       REPEAT
       z:=cadd(csub(cintpower(z,order),cmulti_CR(z,0.5)),c);
       z1:=cadd(csub(cintpower(z,order),cmulti_CR(z2,0.5)),c);
       z2:=z;
       z:=z1;
       inc(iteration);
       try length_z:=z2.x*z2.x+z2.y*z2.y except length_z:=0; end
       UNTIL (iteration > max_iteration) or (length_Z>bailout)
       else
       REPEAT
       z:=cadd(csub(cintpower(z,order),cmulti_CR(z,0.5)),cc);
       z1:=cadd(csub(cintpower(z,order),cmulti_CR(z2,0.5)),cc);
       z2:=z;
       z:=z1;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   phoenix.x:=z.x;
   phoenix.y:=z.y;
   phoenix.z:=length_z;
   phoenix.i:=iteration;
end;

//----------> nova
function nova(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;

BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
   //z(n+1) = ((p-1)*z(n)^p + 1)/(p*z(n)^(p - 1)).
       {z2 = z;
    z1 = z*z;
    z=z-((z*z1-1)/(3*z1))+c;
    z3 = z-z2;}
       if not julia then
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub_CR(cmulti(z,z1),1),
                      cmulti_CR(z1,order))),
               c);
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR (length_z>1e+20)
       else
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub_CR(cmulti(z,z1),1),
                      cmulti_CR(z1,order))),
               cc);
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>1e+20);

   nova.x:=z.x;
   nova.y:=z.y;
   nova.z:=length_z;
   nova.i:=iteration;
end;

//----------> novavarI
function novavarI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;
BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
       if not julia then
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       {z2=z;
        z1=z*z*z;
    	  z=z-(z*z1-z)/(4*z1-1)+c;}
       z:=cadd(
               csub(z,cdiv(
                      csub(cmulti(z,z1),z),
                      csub_CR(cmulti_CR(z1,order),1))),
               c);
       //z:=cadd(cintpower(z,order),cintpower(c,order-1));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint)
       else
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub(cmulti(z,z1),z),
                      csub_CR(cmulti_CR(z1,order),1))),
               c);
       z:=cadd(cintpower(z,order),cintpower(c,order-1));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint);

   novavarI.x:=z.x;
   novavarI.y:=z.y;
   novavarI.z:=length_z;
   novavarI.i:=iteration;
end;

//----------> novavarII
function novavarII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;
BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
       if not julia then
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       {z2=z;
        z1=z*z*z;
    	  z=z-(z*z1-z)/(4*z1-1)+(c*z1)^2;}
       z:=cadd(
               csub(z,cdiv(
                      csub(cmulti(z,z1),z),
                      csub_CR(cmulti_CR(z1,order),1))),
                      cmulti(c,z1));

       //z:=cadd(cintpower(z,order),cintpower(c,order-1));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint)
       else
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub(cmulti(z,z1),z),
                      csub_CR(cmulti_CR(z1,order),1))),
                      cmulti(cc,z1));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint);

   novavarII.x:=z.x;
   novavarII.y:=z.y;
   novavarII.z:=length_z;
   novavarII.i:=iteration;
end;



//----------> newton_mI
function newton_mI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;
BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
       if not julia then
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub_CR(cmulti(z,z1),1),
                      cmulti_CR(z1,order))),
               c);
       z:=cmulti(cintpower(z,order),c);
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR (length_z>1e+20)
       else
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub_CR(cmulti(z,z1),1),
                      cmulti_CR(z1,order))),
               cc);
       z:=cmulti(cintpower(z,order),cc);
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR (length_z>1e+20);

   newton_mI.x:=z.x;
   newton_mI.y:=z.y;
   newton_mI.z:=length_z;
   newton_mI.i:=iteration;
end;

//----------> newton_mII
function newton_mII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;
BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
       if not julia then
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub_CR(cmulti(z,z1),1),
                      cmulti_CR(z1,order))),
               c);
       z:=cadd(cintpower(z,order),cintpower(c,order-1));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR (length_z>1e+20)
       else
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=cadd(
               csub(z,cdiv(
                      csub_CR(cmulti(z,z1),1),
                      cmulti_CR(z1,order))),
               cc);
       z:=cadd(cintpower(z,order),cintpower(c,order-1));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL  (length_z<delta) OR (iteration > max_iteration) OR (length_z>1e+20);

   newton_mII.x:=z.x;
   newton_mII.y:=z.y;
   newton_mII.z:=length_z;
   newton_mII.i:=iteration;
end;

//----------> newtonmIII
function newtonmIII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;
BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     //if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
       if not julia then
       REPEAT
       z2:=z;
       z:=
          csub(z,
               cdiv(
               csub_CR(csub(cmulti(cintpower(z,order),c),cmulti(cintpower(z,order-1),c)),1),
               csub(csub(cmulti_CR(cmulti(cintpower(z,order-1),c),3),cmulti(z,c)),z)
               ));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint)
       else
       REPEAT
       z2:=z;
       z:=
          cadd(cc,
               cdiv(
               csub_CR(csub(cmulti(cintpower(z,order),cc),cmulti(cintpower(z,order-1),cc)),1),
               csub(csub(cmulti_CR(cmulti(cintpower(z,order-1),c),3),cmulti(z,cc)),z)
               ));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint);

   newtonmIII.x:=z.x;
   newtonmIII.y:=z.y;
   newtonmIII.z:=length_z;
   newtonmIII.i:=iteration;
end;

//----------> newtonmiv
function newtonmiv(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,c,cc,z1,z2,z3:tcomplex;
   length_z,delta:float;
BEGIN
     delta:=1/fzehn_hoch(bailout);
     iteration:=0;
     c.x:=ca;
     c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
     cc.x:=const_ext;
     cc.y:=const_imag;
     if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;   z:=c;
     z2.x:=0;
     z2.y:=0;
     z1:=z;
     //if not julia then begin z.x:=1; z.y:=0; end else begin z.x:=c.x+1; z.y:=z.y; end;
       if not julia then
       REPEAT
       z2:=z;
       //z:=cintpower(z,order-1);
       //z=z-(z*z*z*z*c-z+c)/(4*z*z*z*c);
       z:=
          csub(z,
               cdiv(
               csub(cmulti(cintpower(z,order),c),cadd(z,c)),
               cmulti(cmulti_CR(cintpower(z,order-1),order),c)
               ));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint)
       else
       REPEAT
       z2:=z;
       z1:=cintpower(z,order-1);
       z:=
          csub(z,
               cdiv(
               csub(cmulti(cintpower(z,order),cc),cadd(z,cc)),
               cmulti(cmulti_CR(cintpower(z,order-1),order),cc)
               ));
       inc(iteration);
       z3:=csub(z,z2);
       try length_z:=z3.x*z3.x+z3.y*z3.y except length_z:=0; end
       UNTIL (length_z<delta) OR (iteration > max_iteration) OR(length_z>maxint);

   newtonmiv.x:=z.x;
   newtonmiv.y:=z.y;
   newtonmiv.z:=length_z;
   newtonmiv.i:=iteration;
end;

//----------> quartetI
function quartetI(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z2,c,cc:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;

   z:=c;
       if not julia then
       REPEAT                //z2=z; z=z^n+c; c=z2;
       z2:=z;
       z:=cadd(cintpower(z,order),c);
       c:=z2;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z2:=z;
       z:=cadd(cintpower(z,order),cc);
       cc:=z2;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   quartetI.x:=z.x;
   quartetI.y:=z.y;
   quartetI.z:=length_z;
   quartetI.i:=iteration;
end;

//----------> quartetII
function quartetII(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z2,c,cc:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;

   z:=c;
       if not julia then
       REPEAT                //z2=z; z=z^n*log(z)+c c=z2;
       z2:=z;
       z:=cadd(cmulti(cintpower(z,order),clog(z)),c);
       c:=z2;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z2:=z;
       z:=cadd(cmulti(cintpower(z,order),clog(z)),cc);
       cc:=z2;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   quartetii.x:=z.x;
   quartetii.y:=z.y;
   quartetii.z:=length_z;
   quartetii.i:=iteration;
end;

//----------> NEWTEST
function newtest(ca,cbi,const_ext,const_imag:float;
         max_iteration,bailout:integer;
         julia,invert:boolean;order:integer;o_r,o_i:float)                :return;
var
   ITERATION      :integer;
   z,z2,c,cc:tcomplex;
   length_z:float;
BEGIN
   iteration:=0;
   cc.x:=const_ext;
   cc.y:=const_imag;
   c.x:=ca;
   c.y:=cbi; c:=cmulti(c,cmplx(o_r,o_i));
   if invert then begin c:=cintdiv_RC(1,c); cc:=cintdiv_RC(1,cc);  end;

   z:=c;
       if not julia then
       REPEAT                //z2=z; z=(z*c)^n-c*n; c=z2;
       z2:=z;
       z:=cadd(cmulti(cintpower(z,order),clog(z)),c);
       c:=z2;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration)
       else
       REPEAT
       z2:=z;
       z:=cadd(cintpower(z,order),cc);
       cc:=z2;
       inc(iteration);
       length_z:=z.x*z.x+z.y*z.y;
       UNTIL (length_z > bailout) OR (iteration > max_iteration);
   newtest.x:=z.x;
   newtest.y:=z.y;
   newtest.z:=length_z;
   newtest.i:=iteration;
end;

end.
